@@ -1,7 +1,7 @@
 # vim: set ts=8 sw=4 sts=4 et ai tw=79:
 """
 django-pstore -- Python Protected Password Store (Django app)
-Copyright (C) 2012,2013  Walter Doekes <wdoekes>, OSSO B.V.
+Copyright (C) 2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.
 
     This application is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -23,7 +23,6 @@ from logging import getLogger
 
 from django.contrib.auth.decorators import user_passes_test
 from django.core.exceptions import PermissionDenied
-from django.utils.decorators import available_attrs
 
 
 logger = getLogger('pstore.audit')
@@ -33,6 +32,7 @@ def audit_view(description, mutates=False):
     """
     Decorator to log the calling of this view.  Usage::
 
+        @csrf_exempt
         @nonce_required
         @require_POST
         @audit_view('bakes pizza', mutates=True)
@@ -40,13 +40,13 @@ def audit_view(description, mutates=False):
             # ...
     """
     def decorator(func):
-        @wraps(func, assigned=available_attrs(func))
+        @wraps(func)
         def inner(request, *args, **kwargs):
-            humanargs = ', '.join(list(args) +
-                                  ['%s=%s' % (k, v)
-                                   for k, v in kwargs.items()])
+            humanargs = ', '.join(
+                list(args)
+                + ['%s=%s' % (k, v) for k, v in kwargs.items()])
             address = str(request.META.get('REMOTE_ADDR', 'UNKNOWN_IP'))
-            if request.user.is_anonymous():
+            if not request.user.is_authenticated:
                 message = (u'Anonymous user on %s %s %s' %
                            (address, description, humanargs))
             else:
@@ -75,9 +75,11 @@ def is_logged_in_with_nonce(user):
     """
     Raises a 403 error in case the user is not logged in.
     """
-    if user.is_anonymous():
+    if not user.is_authenticated:
         raise PermissionDenied('User is not logged in')
     if not user.used_nonce:
         raise PermissionDenied('User did not log in with a nonce')
     return True
+
+
 nonce_required = user_passes_test(is_logged_in_with_nonce)
