@@ -1,7 +1,7 @@
 # vim: set ts=8 sw=4 sts=4 et ai tw=79:
 """
 django-pstore -- Python Protected Password Store (Django app)
-Copyright (C) 2010,2012,2013  Walter Doekes <wdoekes>, OSSO B.V.
+Copyright (C) 2010,2012,2013,2015,2017  Walter Doekes <wdoekes>, OSSO B.V.
 
     This application is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -22,16 +22,22 @@ from datetime import datetime
 from random import getrandbits
 
 from django.contrib.auth.models import User
+from django.core.validators import RegexValidator
 from django.db import models
 from django.db.models.signals import pre_delete
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from pstorelib.crypt import encrypts
+from pstorelib.exceptions import CryptError
 
-from pstore.db import Model, AsciiField, BlobField
+from pstore.db import Model, ValidationMixin
 from pstore.notify import (notify_object_deletion, notify_publickey_change,
                            notify_user_deletion)
 
+ascii_validator = RegexValidator(
+    regex='^[\t\r\n\x20-\x7e]*$',
+    message=_('Only readable ASCII is allowed: \\x20..\\x7E TAB CR LF'))
+
 
 class PublicKey(Model):
     """
@@ -47,19 +53,17 @@ class PublicKey(Model):
         mI0EULkrQAEEAKU++49M+QfiSTFJjWQ8Yyr+OKa0V90aNGbYNaGvfzlPVHNS+AwR
         ...
 
-    or the obsolete ssh public key:
-
-        ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAIEA0IcrNyQLdVyCgpQtMfv/...
-
     Encrypted Property objects are encrypted using this public key.
     """
-    user = models.OneToOneField(User, related_name='publickey')
-    key = models.TextField(blank=False,
-            help_text=_('The user\'s public key; can be in ssh authorized_key '
-                        'format (OLD) or in PGP PUBLIC KEY format (NEW).'))
-    description = models.CharField(max_length=255, blank=True,
-            help_text=_('Human readable info about the key, e.g. the PGP key '
-                        'uid (Alex Boonstra (TEST) <alex@example.com>).'))
+    user = models.OneToOneField(
+        User, related_name='publickey', on_delete=models.CASCADE)
+    key = models.TextField(
+        blank=False, validators=[ascii_validator],
+        help_text=_('The user\'s public key; in PGP PUBLIC KEY format.'))
+    description = models.CharField(
+        max_length=255, blank=True,
+        help_text=_('Human readable info about the key, e.g. the PGP key '
+                    'uid (Alex B <alex@example.com>).'))
 
     def __init__(self, *args, **kwargs):
         super(PublicKey, self).__init__(*args, **kwargs)
@@ -85,9 +89,7 @@ class PublicKey(Model):
         return changes
 
     def key_type(self):
-        if self.key.startswith('ssh-rsa '):
-            return 'sshrsa'
-        elif self.key.startswith('----'):
+        if self.key.startswith('----'):
             return 'gpg'
         return 'unknown'
 
@@ -103,8 +105,8 @@ class PublicKey(Model):
 
         return ret
 
-    def __unicode__(self):
-        return _(u"%(user)s's %(key_type)s public key") % {
+    def __str__(self):
+        return _("%(user)s's %(key_type)s public key") % {
             'user': self.user, 'key_type': self.key_type()
         }
 
@@ -146,7 +148,8 @@ class Object(Model):
     the "password" for automatic ssh login. The Object itself does not assign
     any meaning to it though.
     """
-    identifier = AsciiField(max_length=255, unique=True)
+    identifier = models.CharField(
+        max_length=255, unique=True, validators=[ascii_validator])
 
     class Meta:
         permissions = (
@@ -154,26 +157,27 @@ class Object(Model):
             ('view_any_object', _('View any object')),
         )
 
-    def __unicode__(self):
+    def __str__(self):
         return self.identifier
 
 
-class ObjectPerm(models.Model):
+class ObjectPerm(ValidationMixin, models.Model):
     """
     Who has read and write permissions for an Object.
     """
-    object = models.ForeignKey(Object, related_name='allowed')
-    user = models.ForeignKey(User)
+    object = models.ForeignKey(
+        Object, related_name='allowed', on_delete=models.CASCADE)
+    user = models.ForeignKey(User, on_delete=models.CASCADE)
     can_write = models.BooleanField()
 
-    def __unicode__(self):
+    def __str__(self):
         dict = {'user': self.user, 'object': self.object}
         if self.can_write:
             return _('%(user)s can write %(object)s') % dict
         return _('%(user)s can read %(object)s') % dict
 
 
-class Property(models.Model):
+class Property(ValidationMixin, models.Model):
     """
     An Object can hold one or more properties. For public properties, there
     is only one record. For private/shared (encrypted) properties, there is one
@@ -194,21 +198,26 @@ class Property(models.Model):
 
     TYPE_CHOICES = (
         (TYPE_PUBLIC, 'Public (unencrypted)'),
-        #FUTURE#(TYPE_PRIVATE, 'Private (encrypted)'),
+        # #FUTURE# (TYPE_PRIVATE, 'Private (encrypted)'),
         (TYPE_SHARED, 'Shared (encrypted)'),
     )
 
     created = models.DateTimeField(auto_now_add=True)
-    object = models.ForeignKey(Object, related_name='properties')
-    name = AsciiField(max_length=255,
-            help_text=_('The property identifier/name.'))
-    type = models.PositiveSmallIntegerField(choices=TYPE_CHOICES,
-            default=TYPE_PUBLIC, help_text=_('Property properties ;-)'))
+    object = models.ForeignKey(
+        Object, related_name='properties', on_delete=models.CASCADE)
+    name = models.CharField(
+        max_length=255,
+        help_text=_('The property identifier/name.'),
+        validators=[ascii_validator])
+    type = models.PositiveSmallIntegerField(
+        choices=TYPE_CHOICES,
+        default=TYPE_PUBLIC, help_text=_('Property properties ;-)'))
 
     # Make sure you defer('value') unless you need it!
-    value = BlobField(blank=True)
+    value = models.BinaryField(blank=True)
 
-    user = models.ForeignKey(User, blank=True, null=True, default=None)
+    user = models.ForeignKey(
+        User, blank=True, null=True, default=None, on_delete=models.CASCADE)
 
     class Meta:
         unique_together = ('object', 'name', 'user')
@@ -228,13 +237,13 @@ class Property(models.Model):
         assert self.user_id is None
         return 'none'
 
-    def __unicode__(self):
+    def __str__(self):
         return _('%(object)s -> %(name)s (%(user)s)') % {
             'object': self.object, 'name': self.name, 'user': self.user
         }
 
 
-class Nonce(models.Model):
+class Nonce(ValidationMixin, models.Model):
     """
     To authenticate users, we use encrypted nonces. We generate a random value,
     encrypt the data using the user's public key and hand out that nonce.
@@ -256,9 +265,9 @@ class Nonce(models.Model):
     MAX_LENGTH = 32     # exactly 32
 
     created = models.DateTimeField(auto_now_add=True)  # expiry depends on this
-    user = models.ForeignKey(User)
-    value = BlobField()
-    encrypted = BlobField()
+    user = models.ForeignKey(User, on_delete=models.CASCADE)
+    value = models.BinaryField()
+    encrypted = models.BinaryField()
 
     @classmethod
     def is_sane(cls, value):
@@ -266,7 +275,7 @@ class Nonce(models.Model):
 
     @classmethod
     def is_sane_b64(cls, value):
-        if not isinstance(value, basestring):
+        if not isinstance(value, (bytes, bytearray)):
             return False
         b64_min_length = int(cls.MIN_LENGTH * 4.0 / 3)
         b64_max_length = int((cls.MAX_LENGTH + 3.0) * 4 / 3)
@@ -282,19 +291,9 @@ class Nonce(models.Model):
 
     def generate(self):
         assert not self.value
-
-        value = []
         bytes = 32
         random = getrandbits(bytes * 8)  # is this random enough?
-
-        # Long-to-bytes in a little endian fashion.
-        while random:
-            value.append('%c' % (random & 0xff),)
-            random >>= 8
-        while len(value) < bytes:
-            value.append('\0')  # most significant bytes 0?
-
-        self.value = ''.join(value)
+        self.value = random.to_bytes(length=bytes, byteorder='little')
         assert self.is_sane(self.value)
 
     def encrypt(self):
@@ -303,7 +302,10 @@ class Nonce(models.Model):
         assert not self.encrypted
 
         publickey = self.user.publickey
-        self.encrypted = encrypts(self.value, public_key=publickey.key)
+        try:
+            self.encrypted = encrypts(self.value, public_key=publickey.key)
+        except CryptError as e:
+            raise CryptError("Encrypting for '%s' failed" % (self.user,), e)
 
     def save(self, **kwargs):
         # Make sure we have a nonce.
@@ -321,7 +323,7 @@ class Nonce(models.Model):
 
 def on_object_delete(instance, **kwargs):
     notify_object_deletion(object=instance)
-pre_delete.connect(on_object_delete, sender=Object)
+pre_delete.connect(on_object_delete, sender=Object)  # noqa
 
 
 def on_user_delete(instance, **kwargs):
@@ -337,4 +339,4 @@ def on_user_delete(instance, **kwargs):
     objects = Object.objects.filter(pk__in=object_pks).order_by('identifier')
 
     notify_user_deletion(user=instance, publickey=publickey, objects=objects)
-pre_delete.connect(on_user_delete, sender=User)
+pre_delete.connect(on_user_delete, sender=User)  # noqa
