@@ -1,7 +1,7 @@
 # vim: set ts=8 sw=4 sts=4 et ai tw=79:
 """
 django-pstore -- Python Protected Password Store (Django app)
-Copyright (C) 2013  Walter Doekes <wdoekes>, OSSO B.V.
+Copyright (C) 2013,2015  Walter Doekes <wdoekes>, OSSO B.V.
 
     This application is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -19,20 +19,21 @@ Copyright (C) 2013  Walter Doekes <wdoekes>, OSSO B.V.
     USA.
 """
 from base64 import b64encode
-from email.Utils import formataddr
+from email.utils import formataddr
 
 from django.contrib.admin.models import DELETION, LogEntry
 from django.contrib.contenttypes.models import ContentType
 from django.conf import settings
 from django.core.mail import send_mail
 from django.db.models import Q
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text
 
 
 def collect_object_info(property_qs, user):
     MAX_PROPERTY_SIZE = 4096
 
     info = []
+    # BEWARE: For the SQLite3 backend, the LENGTH() is incorrect!
     properties = (property_qs.filter(Q(user=None) | Q(user=user))
                   .filter(Q(user=None) | Q(user=user))
                   .defer('value')
@@ -74,7 +75,7 @@ def collect_object_info(property_qs, user):
                               for i in readable.split('\n')).rstrip(),))
         if value is not None:
             info.append('; base64 encoded data')
-            encoded = b64encode(value)
+            encoded = b64encode(value).decode('ascii')
             for i in range(0, len(encoded), 70):
                 info.append('  %s' % (encoded[i:(i + 70)],))
         info.append('')
@@ -117,7 +118,7 @@ def notify_object_deletion(object):
             user=user,  # NOTE: not the *deleting* user
             content_type=ContentType.objects.get_for_model(object),
             object_id=object.pk,
-            object_repr=force_unicode(object),
+            object_repr=force_text(object),
             action_flag=DELETION,
             change_message=flatbody
         )
@@ -128,10 +129,10 @@ def notify_object_deletion(object):
 
         # Send out a mail.
         if settings.DEBUG:
-            print 'Sending mail to:', toaddr
-            print '--'
-            print flatbody
-            print '--'
+            print('Sending mail to: {0}'.format(toaddr))
+            print('--')
+            print(flatbody)
+            print('--')
         else:
             send_mail(subject, flatbody, fromaddr, (toaddr,))
 
@@ -172,10 +173,10 @@ def notify_publickey_change(old_publickey, new_publickey):
 
     # Send out a mail.
     if settings.DEBUG:
-        print 'Sending mail to:', user_toaddrs + admin_toaddrs
-        print '--'
-        print flatbody
-        print '--'
+        print('Sending mail to: {0}'.format(user_toaddrs + admin_toaddrs))
+        print('--')
+        print(flatbody)
+        print('--')
     else:
         # Two separate mails. The user does not need to know who the admins
         # are.
@@ -204,11 +205,12 @@ def notify_user_deletion(user, publickey, objects):
         body.append('- %s' % (object.identifier,))
     body.append('')
 
-    body.append('\nPublic key:\n')
-    body.append('[description]\n  %s\n' %
-                (publickey.description.replace('\n', '\n  '),))
-    body.append('[key]\n  %s' % (publickey.key.replace('\n', '\n  '),))
-    body.append('')
+    if publickey:
+        body.append('\nPublic key:\n')
+        body.append('[description]\n  %s\n' %
+                    (publickey.description.replace('\n', '\n  '),))
+        body.append('[key]\n  %s' % (publickey.key.replace('\n', '\n  '),))
+        body.append('')
 
     for object in objects:
         body.append('\nObject %s:\n' % (object.identifier,))
@@ -219,23 +221,34 @@ def notify_user_deletion(user, publickey, objects):
     # Do we have an admin log? Then store the flatbody there. We don't
     # want it mailed to an insecure location if we can help it.
     ct = ContentType.objects.get_for_model(user)
+    extra_message = ''
     try:
-        entry = LogEntry.objects.get(content_type=ct, object_id=user.id,
-                                     action_flag=DELETION)
+        entry = LogEntry.objects.get(
+            content_type=ct, object_id=user.id,
+            action_flag=DELETION)
+    except LogEntry.MultipleObjectsReturned:
+        # In case we reset the auto_increment, we may be looking at old
+        # entries as well. Take the newest one.
+        entry = LogEntry.objects.filter(
+            content_type=ct, object_id=user.id,
+            action_flag=DELETION).latest('id')
+        extra_message = (
+            '\nWARNING: Multiple LogEntry items found for UID %r\n' % user.pk)
     except LogEntry.DoesNotExist:
         entry = None
-    else:
+
+    if entry:
         entry.change_message += flatbody
         entry.save()
         flatbody = ('User %s was deleted from the pstore!\n\n'
                     'Details about removed objects can be found in the admin '
-                    'log %d.\n' % (user.username, entry.id))
+                    'log %d.\n%s' % (user.username, entry.id, extra_message))
 
     # Send out a mail.
     if settings.DEBUG:
-        print 'Sending mail to:', admin_toaddrs
-        print '--'
-        print flatbody
-        print '--'
+        print('Sending mail to: {0}'.format(admin_toaddrs))
+        print('--')
+        print(flatbody)
+        print('--')
     else:
         send_mail(subject, flatbody, fromaddr, admin_toaddrs)
