@@ -1,7 +1,7 @@
 # vim: set ts=8 sw=4 sts=4 et ai tw=79:
 """
 django-pstore -- Python Protected Password Store (Django app)
-Copyright (C) 2012,2013  Walter Doekes <wdoekes>, OSSO B.V.
+Copyright (C) 2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.
 
     This application is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -47,7 +47,7 @@ def validate_nonce_b64(nonce_b64):
         raise PermissionDenied('Is not a nonce')
 
     # Re-add padding.
-    nonce_b64 += (2 - ((len(nonce_b64) + 2) % 3)) * '='
+    nonce_b64 += (2 - ((len(nonce_b64) + 2) % 3)) * b'='
     try:
         nonce = b64decode(nonce_b64)
     except TypeError:
