@@ -1,7 +1,7 @@
 # vim: set ts=8 sw=4 sts=4 et ai tw=79:
 """
 django-pstore -- Python Protected Password Store (Django app)
-Copyright (C) 2012,2013  Walter Doekes <wdoekes>, OSSO B.V.
+Copyright (C) 2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.
 
     This application is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -19,7 +19,9 @@ Copyright (C) 2012,2013  Walter Doekes <wdoekes>, OSSO B.V.
     USA.
 """
 from datetime import datetime, timedelta
-from os import chmod
+from io import BytesIO
+from os import chmod, rename, unlink
+from os.path import basename
 
 from django.contrib.auth.models import User
 from django.core.exceptions import PermissionDenied
@@ -27,13 +29,13 @@ from django.db import connection
 from django.db.models import Q
 from django.http import Http404
 from django.shortcuts import get_object_or_404
+from django.views.decorators.csrf import csrf_exempt
 from django.views.decorators.http import require_GET, require_POST
 
-from pstorelib.bytes import BytesIO
 from pstorelib.server import urlunquote
 
 from pstore.decorators import audit_view, nonce_required
-from pstore.http import EncryptedResponse, VoidResponse
+from pstore.http import EncryptedResponse, HttpError, VoidResponse
 from pstore.models import Nonce, Object, ObjectPerm, Property
 from pstore.security import get_object_or_403
 
@@ -50,10 +52,12 @@ def create_property(object, property, file, user):
     if user:
         ptype = Property.TYPE_SHARED
         if file.size == 0:
-            raise Exception('FIXME-EXCEPTION: 0len = impossible')
+            raise HttpError(422, 'encrypted properties cannot have 0 length')
     else:
         ptype = Property.TYPE_PUBLIC
 
+    # Depending on the file size, we either get an InMemoryUploadedFile or a
+    # TemporaryUploadedFile.
     if hasattr(file, 'temporary_file_path'):
         # Tricks! We'll use a custom INSERT on localhost. Otherwise we'll
         # run into the MySQL max_packet_size limit.
@@ -63,7 +67,9 @@ def create_property(object, property, file, user):
         tempname = None
         data = file.read()
         if len(data) != file.size:
-            raise RuntimeError('file read returned wrong amount of bytes')
+            raise HttpError(500, 'file read returned wrong amount of bytes', (
+                'We expected to read %d bytes at once from %s. We only got '
+                '%d bytes.' % (file.size, file.name, len(data))))
 
     prop = Property.objects.create(object=object, name=property,
                                    type=ptype, value=data, user=user)
@@ -74,13 +80,28 @@ def create_property(object, property, file, user):
         # We use LOAD_FILE() because it speeds up the writing by a factor
         # three. We expect you to run the MySQLd on localhost for now..
         if (connection.settings_dict['HOST'] not in
-            ('', 'localhost', '127.0.0.1')):
-            raise Exception('FIXME-EXCEPTION: only works on localhost')
+                ('', 'localhost', '127.0.0.1')):
+            raise HttpError(413, 'request too large (mysqld infrastructure)', (
+                'mysqld can only use LOAD_FILE() on localhost and the DB '
+                'server seems to be running on %s' % (
+                    connection.settings_dict['HOST'],)))
+
         # MySQLd must get read powers.
+        # @@secure_file_priv == '/var/lib/mysql-files/'
+        # That directory must be set to: mysql:www-data 0730.
         try:
-            chmod(tempname, 0644)
-        except:
-            raise Exception('FIXME-EXCEPTION: chmod failed for LOAD_FILE')
+            chmod(tempname, 0o604)
+            newname = '/var/lib/mysql-files/%s.%s.%s.%s' % (
+                object, property, user, basename(tempname))
+            # Hope it's on the same filesystem...
+            rename(tempname, newname)
+        except Exception as e:
+            raise HttpError(413, 'request too large (webserver permissions)', (
+                'For mysqld to do load LOAD_FILE() on %s, we need to alter '
+                'file permissions, we got: %s' % (tempname, e)))
+        else:
+            tempname = newname
+
         # Try to read the file.
         try:
             cursor = connection.cursor()
@@ -88,18 +109,34 @@ def create_property(object, property, file, user):
                 UPDATE pstore_property SET value = LOAD_FILE(%s)
                 WHERE id = %s;
             ''', (tempname, prop.id))
-        except:
-            raise Exception('FIXME-EXCEPTION: mysqld LOAD_FILE failed for'
-                            ' %s, check apparmor. Check File_Priv mysql '
-                            'permissions, check @@max_allowed_packet, '
-                            'checkl @@secure_file_priv' % (tempname,))
+        except Exception as e:
+            # Tip #1:
+            #   mysql> UPDATE mysql.user SET file_priv = 'Y'
+            #          WHERE user = 'pstore' AND host = 'localhost'
+            #          AND file_priv = 'N'; FLUSH PRIVILEGES;
+            #
+            # Tip #2:
+            #   /etc/mysql/my.cnf:
+            #     # If you're doing replication, you must use MIXED or
+            #     # ROW based replication. Otherwise LOAD_FILE will fail.
+            #     binlog_format = MIXED
+            #
+            raise HttpError(413, 'request too large (mysqld permissions)', (
+                'mysqld LOAD_FILE failed for %s, when writing %s->%s (@%s). '
+                'Check apparmor/@@secure_file_priv, '
+                'check File_Priv mysql permissions, '
+                'check @@max_allowed_packet: %s ' % (
+                    tempname, object, property, user, e)))
         finally:
+            # Remove access to the file asap.
+            unlink(tempname)
             # Close the cursor here. The http middleware will commit the
-            # transaction when done.
+            # transaction when/if done.
             cursor.close()
 
 
 # NOT nonce_required.. obviously..
+@csrf_exempt
 @require_POST
 @audit_view('creates nonce')
 def create_nonce(request):
@@ -147,26 +184,31 @@ def get_property(request, object_identifier, property_name):
     # Query:
     qs = Property.objects.filter(
         object__identifier=object_identifier,
-        name=property_name
-    )
+        name=property_name)
     if u:
         qs = qs.filter(Q(user=None) | Q(user__username=u))
     else:
         qs = qs.filter(user=None)
 
     # Check authorization:
-    if request.user.has_perms('pstore.view_any_object'):
+    if request.user.has_perm('pstore.view_any_object'):
         pass
     elif not ObjectPerm.objects.filter(
-        object__identifier=object_identifier,
-        user=request.user
-    ).exists():
+            object__identifier=object_identifier,
+            user=request.user).exists():
         raise PermissionDenied('Not staff and not permitted')
 
     # Results:
     items = list(qs[0:2])
     if not items:
-        raise Http404('No such property')
+        # Was this because we didn't have permission or because there
+        # simply wasn't a property?
+        if (request.user.has_perm('pstore.view_any_object')
+                and not Property.objects.filter(
+                    object__identifier=object_identifier,
+                    name=property_name).exists()):
+            raise Http404('No such property')
+        raise PermissionDenied('Not staff or not permitted')
 
     if len(items) > 1:
         class_ = Property.MultipleObjectsReturned
@@ -183,6 +225,7 @@ def get_property(request, object_identifier, property_name):
     return EncryptedResponse(fp=file, enctype=property.enctype())
 
 
+@csrf_exempt
 @nonce_required
 @require_POST
 @audit_view('sets or replaces a property', mutates=True)
@@ -192,8 +235,9 @@ def set_property(request, object_identifier, property_name):
     # Check the rest of the arguments.
     if len(request.GET) != 0:
         raise NotImplementedError('Unexpected GET args', request.GET)
-    if len(request.POST) != 1:  # nonce was here..
+    if set(request.POST.keys()) - set(['nonce_b64', 'o_excl']):
         raise NotImplementedError('Unexpected POST args', request.POST)
+    o_excl = (request.POST.get('o_excl', '') == '1')  # force create or fail
 
     # Docode URI arguments.
     object_identifier = urlunquote(object_identifier)
@@ -219,14 +263,22 @@ def set_property(request, object_identifier, property_name):
 
     # Autocreate object if it didn't exist yet. Next, make *sure* that 'author'
     # that we found in the nonce is a valid writer.
-    obj, created = Object.objects.get_or_create(identifier=object_identifier)
+    if users:
+        obj, created = Object.objects.get_or_create(
+            identifier=object_identifier)
+    else:
+        try:
+            obj = Object.objects.get(identifier=object_identifier)
+        except Object.DoesNotExist:
+            raise Http404('Refusing to create an object automatically')
+        else:
+            created = False
+
     if created:
-        # We did get a proper list of users, right?
-        if not users:
-            # We need a list of users if this is a new entry! Yes.. that means
-            # that we *must* have an encrypted property for the machine to be
-            # created. Let that be the case for now.
-            raise Exception('FIXME-EXCEPTION')
+        # We need a list of users if this is a new entry! Yes.. that means
+        # that we *must* have an encrypted property for the machine to be
+        # created. Let that be the case for now.
+        assert users
 
         # Author must be in the list of users too.
         if author not in users:
@@ -239,6 +291,9 @@ def set_property(request, object_identifier, property_name):
             ObjectPerm.objects.create(object=obj, user=user, can_write=True)
 
     else:
+        if o_excl:
+            raise PermissionDenied('Object exists already')
+
         # Check that author has write powers.
         # TODO: we could allow SU-users to do stuff..
         if not obj.allowed.filter(can_write=True, user=author).exists():
@@ -275,6 +330,7 @@ def set_property(request, object_identifier, property_name):
     return VoidResponse()
 
 
+@csrf_exempt
 @nonce_required
 @require_POST
 @audit_view('updates properties and/or permissions', mutates=True)
@@ -305,8 +361,8 @@ def update_properties(request, object_identifier):
             username = file.name
             usernames.append(username)
     usernames = set(usernames)
-    users = dict((i.username, i) for i in
-                 User.objects.filter(username__in=usernames))
+    users = dict([(i.username, i) for i in
+                  User.objects.filter(username__in=usernames)])
     if len(users) != len(usernames):
         raise Exception('FIXME-EXCEPTION: user count invalid')
 
@@ -339,6 +395,7 @@ def update_properties(request, object_identifier):
     currently_allowed = set([i.user for i in
                              (ObjectPerm.objects.filter(object=obj)
                               .select_related('user'))])
+    del currently_allowed  # fixme.. use this
     # XXX: take currently_allowed, remove now-allowed:
     # delete the leftovers
     # add the not-in-currently_allowed
