@@ -1,7 +1,7 @@
 # vim: set ts=8 sw=4 sts=4 et ai tw=79:
 """
 django-pstore -- Python Protected Password Store (Django app)
-Copyright (C) 2012,2013  Walter Doekes <wdoekes>, OSSO B.V.
+Copyright (C) 2012,2013,2015  Walter Doekes <wdoekes>, OSSO B.V.
 
     This application is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -24,15 +24,16 @@ from collections import defaultdict
 from django.contrib.auth.models import User
 from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
 from django.db.models import Q
+from django.http import JsonResponse
 from django.shortcuts import get_object_or_404
 from django.views.decorators.http import require_GET
 
 from pstorelib.server import urlunquote
 
 from pstore.decorators import audit_view, nonce_required
+from pstore.http import HttpError
 from pstore.models import Object, ObjectPerm, PublicKey, Property
 from pstore.security import get_object_or_403
-from pstore.xhr import JsonResponse
 
 
 @nonce_required
@@ -45,13 +46,11 @@ def get_object(request, object_identifier):
     Note that if you want info about the properties for a different user,
     you'll need to redo the query with that user id.
     """
-    # Query strings:
-    u = request.GET.get('u', None)
-
     # Decode object_identifier.
     object_identifier = urlunquote(object_identifier)
 
     # Check authorization and existence:
+    u = request.GET.get('u', None)
     if request.user.has_perm('object.view_any_object'):
         obj = get_object_or_404(Object, identifier=object_identifier)
     elif u != request.user.username:
@@ -59,25 +58,20 @@ def get_object(request, object_identifier):
     else:
         obj = get_object_or_403(Object, identifier=object_identifier)
 
-    # Complicated by the permissions checks: we want the any_perm user to see
-    # all shared and public properties, but not the contents. At the bottom,
-    # the shared properties are added if no user lookup was supplied.
+    # Check if the user is allowed to view this.
     try:
         obj.allowed.get(user=request.user)
     except ObjectDoesNotExist:
         if not request.user.has_perm('object.view_any_object'):
             raise PermissionDenied()
-        is_allowed = False
-    else:
-        is_allowed = True
 
     # Get lots of info for this object.
     result = {}
 
     # Get the allowed users.
     allowed = obj.allowed.select_related('user')
-    result['users'] = dict((i.user.username, {'can_write': i.can_write})
-                           for i in allowed)
+    result['users'] = dict([(i.user.username, {'can_write': i.can_write})
+                            for i in allowed])
 
     # Get a list of properties.
     propqs = Property.objects.filter(object=obj)
@@ -85,13 +79,21 @@ def get_object(request, object_identifier):
         propqs = propqs.filter(Q(user__username=u) | Q(user=None))
     else:
         propqs = propqs.filter(user=None)
+    # BEWARE: For the SQLite3 backend, the LENGTH() is incorrect!
     propqs = propqs.extra(select={'size': 'LENGTH(value)'})
     properties = set(propqs.values_list('id', 'name', 'type', 'size', 'user'))
 
     # Get the values, but only for small properties.
     small_property_ids = [i[0] for i in properties if i[3] <= 2048]
-    property_values = dict(Property.objects.filter(id__in=small_property_ids)
-                           .values_list('id', 'value'))
+
+    # https://code.djangoproject.com/ticket/9619
+    # We cannot do values_list when using SQLite3.
+    # #property_values = dict(Property.objects
+    # #                       .filter(id__in=small_property_ids)
+    # #                       .values_list('id', 'value'))
+    property_qs = Property.objects.filter(id__in=small_property_ids)
+    property_values = dict((i.id, i.value)
+                           for i in property_qs.only('id', 'value'))
 
     # Put properties in the results dictionary.
     result['properties'] = {}
@@ -116,7 +118,8 @@ def get_object(request, object_identifier):
 
         # If data is small enough, it is set too.
         if property_id in property_values:
-            info['data'] = b64encode(property_values[property_id])
+            info['data'] = b64encode(
+                property_values[property_id]).decode('ascii')
 
         result['properties'][name] = info
 
@@ -124,7 +127,8 @@ def get_object(request, object_identifier):
     # those.
     if not u:
         # Fetch property names for the properties.
-        for name, type in (Property.objects.exclude(user=None)
+        for name, type in (Property.objects.filter(object=obj)
+                           .exclude(user=None)
                            .values_list('name', 'type').distinct()):
             assert type == Property.TYPE_SHARED
 
@@ -134,18 +138,13 @@ def get_object(request, object_identifier):
 
             result['properties'][name] = info
 
-    return JsonResponse(request, result)
+    return JsonResponse(result)
 
 
 @nonce_required
 @require_GET
 @audit_view('lists objects+info')
 def list_objects(request):
-    # Query strings:
-    q = request.GET.get('q', None)  # filter against identifier
-    u = request.GET.get('u', None)  # filter against allowed (allow multiple?)
-    v = request.GET.get('v', None) and True  # list more info
-
     # Check authorization and existence:
     u = request.GET.get('u', None)
     if request.user.has_perm('object.view_any_object'):
@@ -153,6 +152,10 @@ def list_objects(request):
     elif u != request.user.username:
         raise PermissionDenied()
 
+    # Query strings:
+    q = request.GET.get('q', None)  # filter against identifier
+    v = request.GET.get('v', None) and True  # list more info
+
     # Query:
     qs = Object.objects.all()  # no ordering here, that's up to the client
     if q:
@@ -183,7 +186,70 @@ def list_objects(request):
         result = dict((i, True)
                       for i in qs.values_list('identifier', flat=True))
 
-    return JsonResponse(request, result)
+    return JsonResponse(result)
+
+
+@nonce_required
+@require_GET
+@audit_view('search properties')
+def search_properties(request):
+    # Check authorization and existence:
+    u = request.GET.get('u', None)
+    if request.user.has_perm('object.view_any_object'):
+        pass
+    elif u != request.user.username:
+        raise PermissionDenied()
+
+    # Query:
+    qs = Property.objects.all()  # no ordering here, that's up to the client
+    if u:
+        qs = qs.filter(object__allowed__user__username=u)
+
+    # More queries (observe that "icontains" search depends on the DB
+    # backend to work insensitively; MySQL blob does *not* work).
+    if 'propkey_icontains' in request.GET:
+        qs = qs.filter(name__icontains=request.GET['propkey_icontains'])
+    if 'propvalue_icontains' in request.GET:
+        # Yuck. That length should've been memoized.
+        qs = (qs
+              .filter(type=Property.TYPE_PUBLIC,
+                      value__icontains=request.GET['propvalue_icontains'])
+              .extra(where=['LENGTH(value) <= 2048']))
+
+    # First sanity check:
+    if qs.count() > 100:
+        # RFC4918 11.2.  422 Unprocessable Entity
+        # > ... For example, this error condition may occur if an XML
+        # > request body contains well-formed (i.e., syntactically
+        # > correct), but semantically erroneous, XML instructions.
+        # In this case, semantics being: be more specific.
+        raise HttpError(422, 'too many results')
+
+    # Fetch extra properties we need:
+    qs = qs.extra(select={'size': 'LENGTH(value)'})
+    qs = qs.select_related('object')
+    qs = qs.values_list('id', 'object__identifier', 'name', 'type', 'size')
+
+    machines = {}
+    for id_, identifier, propkey, proptype, size in qs:
+        propvalue = None
+        if proptype == Property.TYPE_PUBLIC and size <= 2048:
+            propvalue = (Property.objects.filter(id=id_)
+                         .values_list('value', flat=True)[0])
+
+        info = {
+            'data': propvalue,
+            'size': size,
+            'enctype': ('none', 'any')[proptype != Property.TYPE_PUBLIC],
+        }
+        if propvalue:
+            info['data'] = b64encode(propvalue).decode('ascii')
+
+        if identifier not in machines:
+            machines[identifier] = {'properties': {}}
+        machines[identifier]['properties'][propkey] = info
+
+    return JsonResponse(machines)
 
 
 @require_GET
@@ -208,7 +274,7 @@ def list_users(request):
             'description': object.description,
         }
 
-    return JsonResponse(request, result)
+    return JsonResponse(result)
 
 
 @nonce_required
@@ -231,13 +297,13 @@ def validate(request):
         ))
 
     # FIXME: code below is broken for objects with both readers and writers..
-    ## Assert that all objects have someone who can write.
-    #for object in Object.objects.filter(allowed__can_write=False):
-    #    errors.append((
-    #        'Object', object.id,
-    #        '%s does not have an admin' % (object.identifier,),
-    #        'Add ObjectPerm items with can_write powers',
-    #    ))
+    # # Assert that all objects have someone who can write.
+    # #for object in Object.objects.filter(allowed__can_write=False):
+    # #   errors.append((
+    # #       'Object', object.id,
+    # #       '%s does not have an admin' % (object.identifier,),
+    # #       'Add ObjectPerm items with can_write powers',
+    # #   ))
 
     # Check property types.
     for property in (Property.objects.exclude(type__in=[tpub, tshar])
@@ -280,9 +346,9 @@ def validate(request):
         for property in shared_properties:
             per_property[property.name].append(property.user_id)
 
-        for property_name, property_user_ids in per_property.iteritems():
-            if (len(property_user_ids) != len(user_ids) or
-                set(property_user_ids) != set(user_ids)):
+        for property_name, property_user_ids in per_property.items():
+            if (len(property_user_ids) != len(user_ids)
+                    or set(property_user_ids) != set(user_ids)):
                 errors.append((
                     'Object', object.id,
                     ('%s -> %s has wrong number of properties' %
@@ -291,4 +357,4 @@ def validate(request):
                      'properties for this object.'),
                 ))
 
-    return JsonResponse(request, {'errors': errors})
+    return JsonResponse({'errors': errors})
